# select_on_medians.praat
# djmw 20060308
# djmw 20060323 Added redundancy measure

form Selection on medians
  choice Segment 2
  button 1
  button 2
  button 3
  button 1+2+3
  choice Formant_frequency_scale 3
  button Log10
  button Barks
  button Linear
  comment Frequency scale influences the medians (log(2) = 0.3)
  positive Median_low 0.5
  positive Median_high 1.5
  positive Number_of_segments 6
  comment Last segment spans whole dataset
  comment log10(1.5) = 0.167
endform

# for log scales take 0.7-1.3 as median interval

include save.praat

datadir$ = "/home/david/tex/thesis/data/"

if median_high <= median_low
  exit median_low must be < median_high
endif

call cca_analysis

procedure cca_analysis
  call create_result_table
  call dependency_medianselection_cca m
  call dependency_medianselection_cca f
  select result
  if formant_frequency_scale = 1
    ext$ = "_log"
  elsif formant_frequency_scale = 2
    ext$ = "_barks"
  else
    ext$ = "_linear"
  endif
  Write to short text file... 'savedir$'cca_analysis_results'ext$'.TableOfReal
endproc

procedure dependency_medianselection_cca .gender$
  .file$ = "analyse_bff_"+"'.gender$'"
  Read from file... 'datadir$''.file$'.TableOfReal
  .tor = selected ("TableOfReal")
  if formant_frequency_scale = 1
    printline --- Log10 frequency scale ----
    Formula... if col >= 55 and col <= 63 and self > 0 then log10(self) else self fi
  elsif formant_frequency_scale = 2
    printline --- Bark frequency scale ----
    Formula... if col >= 55 and col <= 63 and self > 0 then hertzToBark(self) else self fi
  else
     printline --- Linear frequency scale ----
  endif
  printline ---- 'datadir$''.file$'.TableOfReal ----
  
  for .is to number_of_segments
    .mmin = 1 -  (1 - median_low) * .is / (number_of_segments-1)
    .mmax = 1 + (median_high - 1) * .is / (number_of_segments-1)
	if .is = number_of_segments
	  .mmin = 0
	  .mmax = 1000
	endif
    call selections_on_medians '.gender$' .tor .mmin .mmax
  endfor
  select .tor
  Remove
endproc

procedure selections_on_medians .gender$ .tor .mmin .mmax
  select .tor
  if segment <= 3
    .ibf = 55 + (segment - 1) * 3
    .ief = .ibf + 3 - 1
  elsif segment = 4
    .ibf = 55
    .ief = .ibf + 3 * 3 - 1
  endif
  Extract column ranges... '.ibf':'.ief'
  .ncols = Get number of columns
  .nrows = Get number of rows
  Rename... cr
  To TableOfReal (medians by row labels)... y
  Rename... crm
  select TableOfReal cr
  Formula... self/TableOfReal_crm[]
  select .tor
  for .i to .nrows
    .select = 1
    for .j to .ncols
      if TableOfReal_cr[.i,.j] < .mmin or TableOfReal_cr[.i,.j] > .mmax
        .select = 0
      endif
   endfor
   Set value... .i 64 .select
  endfor    
  Extract rows where... self[row,64] > 0
  .tmp = selected ("TableOfReal")
  .nrows2 = Get number of rows
  printline --- selected '.nrows2' from '.nrows' rows
  # exchange f and bf's
  call exchange_table .tmp
  .torex = selected ("TableOfReal")
  select .tmp
  plus TableOfReal cr
  plus TableOfReal crm
  Remove
  printline ---- '.mmin' < medians < '.mmax' ----
  select result
  Set value... irow 1 .nrows2
  Set value... irow 2 .mmin
  Set value... irow 3 .mmax
  Set row label (index)... irow '.gender$'
  call get_ccas .torex
  select result
  for .i to 6
    Set value... irow 3+.i cc'.i'
    Set value... irow 5*3+.i vf'.i'
  endfor
  Set value... irow 7*3+1 rdy
  Set value... irow 7*3+2 rdx
  call get_ccas_pca .torex
  select result
  for .i to 6
    Set value... irow 3*3+.i cc'.i'
  endfor
  call discriminant .torex
  select result
  Set value... irow ncolsr-3 fcfyy
  Set value... irow ncolsr-2 fcbyy
  Set value... irow ncolsr-1 fcfny
  Set value... irow ncolsr-0 fcbny
  irow += 1
  select .torex
  Remove
endproc
 
procedure exchange_table .tor
  if segment <= 3
    .ibf = 55 + (segment - 1) * 3
    .ief = .ibf + 3 - 1
    .ibb = 1 + (segment - 1) * 18
    .ieb = .ibb + 18 - 1
  elsif segment = 4
    .ibf = 55
    .ief = .ibf + 3 * 3 - 1
    .ibb = 1
    .ieb = .ibb + 3 * 18 - 1
  endif
  select .tor
  Extract column ranges... '.ibf':'.ief' '.ibb':'.ieb'
  tor_selection = selected ("TableOfReal")
endproc

procedure create_result_table
  ncolsr = 3 + 4*3 + 2*3 + 2 +2+2
  Create TableOfReal... results 2*number_of_segments ncolsr
  Set column label (index)... 1 n
  Set column label (index)... 2 mmin
  Set column label (index)... 3 mmax
  for .i to 3
    Set column label (index)... 1*3+.i F'.i'B18
    Set column label (index)... 2*3+.i F1:3B18__'.i'_
    Set column label (index)... 3*3+.i F'.i'PC3
    Set column label (index)... 4*3+.i F1:3PC3__'.i'_
    Set column label (index)... 5*3+.i vfy__'.i'_
    Set column label (index)... 6*3+.i vfx__'.i'_
  endfor
  Set column label (index)... 7*3+1 rdy
  Set column label (index)... 7*3+2 rdx
  Set column label (index)... ncolsr-3 disf3yy
  Set column label (index)... ncolsr-2 disb18yy
  Set column label (index)... ncolsr-1 disf3ny
  Set column label (index)... ncolsr-0 disb18ny
  irow = 1
  result = selected ("TableOfReal")
endproc

# F1B1:18 F2B18 F3B18 F1:3B18_1 F1:3B18_2 F1:3B18_3
procedure get_ccas .tor
  select .tor
  ncols = Get number of columns
  if ncols != 21
    exit Number of columns ('ncols') must be 21=3+18
  endif
  # F1 vs B1-18
  for .ifor to 3
    select .tor
    Extract column ranges... '.ifor' 4:21
    .fi = selected ("TableOfReal")
    To CCA... 1
    .cca = selected ("CCA")
    .cc = Get correlation... 1
    cc'.ifor' = .cc
    printline '.cc' (correlation of f__'.ifor' with b__1..18_)
    plus .fi
    Remove
  endfor
  select .tor
  To Correlation
  .cor = selected ("Correlation")
  select .tor
  To CCA... 3
  .cca = selected ("CCA")
  for .icc to 3
    select .cca
    .cc = Get correlation... .icc
    printline '.cc' (correlation '.icc' of f__123_ with b__1..18_)
    .i = .icc+3
    cc'.i' = .cc
    plus .cor
    .vf = Get variance fraction... y .icc .icc
    vf'.icc' = .vf
    .vf = Get variance fraction... x .icc .icc
    vf'.i' = .vf   
  endfor
  select .cca
  plus .cor
  rdy = Get redundancy (sl)... y 1 3
  rdx = Get redundancy (sl)... x 1 3
  Remove
endproc

# F1PC1:3 F2PC1:3 F3PC1:3 F1:3PC1:3_1 F1:3PC1:3_2 F1:3PC1:3_3
procedure get_ccas_pca .tor
  select .tor
  ncols = Get number of columns
  if ncols != 21
    exit Number of columns ('ncols') must be 21=3+18
  endif
  # f1-3
  Extract column ranges... 1:3
  .torf = selected ("TableOfReal")
  #bf1-18
  select .tor
  Extract column ranges... 4:21
  .tmp = selected ("TableOfReal")
  To PCA
  .pca = selected ("PCA")
  plus .tmp
  To Configuration... 3
  .tmpc =  selected ("Configuration")
  To TableOfReal
  .tmpt = selected ("TableOfReal")
  plus .torf
  Append columns
  .tor3_3 = selected ("TableOfReal")
  # F1 vs B1-18
  for .ifor to 3
    select .tor3_3
    Extract column ranges... '.ifor' 4:6
    .fi = selected ("TableOfReal")
    To CCA... 1
    .cca = selected ("CCA")
    .cc = Get correlation... 1
    printline '.cc' (correlation of f__'.ifor' with pc__1..3_)
    cc'.ifor' = .cc
    plus .fi
    Remove
  endfor
  select .tor3_3
  To CCA... 3
  for .icc to 3
    .cc = Get correlation... .icc
    printline '.cc' (correlation '.icc' of f__123_ with pc__123_)
    .i = .icc+3
    cc'.i' = .cc
  endfor
  plus .pca
  plus .torf
  plus .tmpc
  plus .tmp
  plus .tmpt
  plus .tor3_3
  Remove
endproc

procedure discriminant .tor
  select .tor
  ncols = Get number of columns
  if ncols != 21
    exit Number of columns ('ncols') must be 21=3+18
  endif
  # formants
  Extract column ranges... 1:3
  .torf = selected ("TableOfReal")
  call get_percentage_correct .torf
  fcfyy = fcyy
  fcfny = fcny  
  select .tor
  Extract column ranges... 4:21
  .torb = selected ("TableOfReal")
  call get_percentage_correct .torb
  fcbyy = fcyy
  fcbny = fcny  
  select .torf
  plus .torb
  Remove
endproc

procedure get_percentage_correct .tor
  select .tor
  To Discriminant
  .dis = selected ("Discriminant")
  plus .tor
  To ClassificationTable... y y
  .ct = selected ("ClassificationTable")
  To Confusion
  .cf = selected ("Confusion")
  fcyy = Get fraction correct
  select .tor
  plus .dis
  To ClassificationTable... n y
  .ct2 = selected ("ClassificationTable")
  To Confusion
  fcny = Get fraction correct
  plus .ct2
  plus .cf
  plus .ct
  plus .dis
  Remove
endproc

procedure put_latex_tables file$
  Read from file...'savedir$''file$'.TableOfReal
  .tor = selected ("TableOfReal")
  nrows = Get number of rows
  for .i to nrows
    

  endfor
endproc

  
