# bias-adaptation-test.praat
# djmw 20040427
# djmw 20040604 added form + different input set
# djmw 20041104 make adaptation of hidden layer possible
# djmw 20041115 Adapt both layers (one output table)
# djmw 20041119 Reorder columns
# djmw 20050319 Changed train/test sets
# djmw 20050324 Removed a bug that caused the wrong test data be tested

numberOfNodes_max=9
biasreset = 0
test = 0

form Bias-adaptation-test
  comment Select from 630 speakers for training
  comment 438 (326+112) males, 192 (136+56) females
  comment indices m(1, 327), f(439,575)
  natural Speaker_train1 1
  natural Speaker_train2 10
  natural Speaker_test1 1
  natural Speaker_test2 630
  natural Sequence_number 1
endform

if test = 1
  number_of_iterations_train = 100
  number_of_iterations_test = 10
  number_of_repetitions = 2
else
  number_of_iterations_train = 1000
  number_of_iterations_test = 100
  number_of_repetitions = 10
endif

procedure preprocess_mf table_id
  select table_id
  Extract row ranges... 1:8078
  Centre columns
  mc = selected ("TableOfReal")
  select table_id
  Extract row ranges... 8079:11579
  Centre columns
  fc = selected ("TableOfReal")
  plus mc
  Append
  tablec = selected ("TableOfReal")
  Formula... (self+80)/160
  
endproc

date$ = "20050324_'sequence_number'"
home$ = "/home/david/"
datadir$ = home$+"tex/thesis/data/"
savedir$ = home$+"tex/thesis/nn-bias2/"
include /home/david/tex/praat_procedures.praat

name$ = "barks_total_frame2_mf_meansperspeaker"

# Read 9 dimensional table

Read from file... 'datadir$''name$'_pc9.TableOfReal
table_tmp = selected ("TableOfReal")
call preprocess_mf table_tmp
table_total = tablec

# Read speaker indices

Read from file... 'datadir$''name$'_index.TableOfReal
speakerindex = selected ("TableOfReal")
nspeakers = Get number of rows

# Check given speaker indices

if speaker_train1 > speaker_train2 or speaker_train2 > nspeakers
  exit Speaker_train1 and Speaker_train2 should be in [1, 'nspeakers']
endif
if speaker_test1 > speaker_test2 or speaker_test2 > nspeakers 
  exit Speaker_test1 and Speaker_test2 should be in [1, 'nspeakers']
endif

row_train1 = Get value... speaker_train1 1
row_train2 =  Get value... speaker_train2 2
nitems_train = row_train2 - row_train1 + 1
nspeakers_train = speaker_train2 - speaker_train1 + 1
printline row_train1 = 'row_train1', row_train2 = 'row_train2', 
  ... nitems_train = 'nitems_train', nspeakers_train = 'nspeakers_train'

row_test1 = Get value... speaker_test1 1
row_test2 =  Get value... speaker_test2 2
nitems_test = row_test2 - row_test1 + 1
nspeakers_test = speaker_test2 - speaker_test1 + 1
printline row_test1 = 'row_test1', row_test2 = 'row_test2', 
  ... nitems_test = 'nitems_test', nspeakers_test = 'nspeakers_test'

#pause
speakerstring$ = "'speaker_train1'"+"_'speaker_train2'"+"_'speaker_test1'"+
  ... "_'speaker_test2'"

call logfile_open bias_adaptation_'speakerstring$'


# split off the items from the selected speakers to train the neural net

select table_total
ncols = Get number of columns
To Pattern and Categories... row_train1 row_train2 0 0
pattern_train = selected ("Pattern")
categories_train = selected ("Categories")

tableoffset = 4
ncols_result = tableoffset + (1+2)*numberOfNodes_max
Create TableOfReal... result nspeakers_test*number_of_repetitions ncols_result
result = selected ("TableOfReal")
Set column label (index)... 1 Begin
Set column label (index)... 2 End
Set column label (index)... 3 Speaker
Set column label (index)... 4 Iter
icol = tableoffset
for nodes to numberOfNodes_max
  icol += 1
  Set column label (index)... icol H'nodes'
  icol += 1
  Set column label (index)... icol H'nodes'L1
  icol += 1
  Set column label (index)... icol H'nodes'L2
endfor

procedure get_fractioncorrect ffnet_id pattern_id categories_id
  select ffnet_id
  plus pattern_id
  To Categories... Winner-takes-all
  categories_2tmp = selected ("Categories")
  plus categories_id
  f_tmp = Get fraction different
  fractionCorrect = 1 - f_tmp
  select categories_2tmp
  Remove
endproc

for i to number_of_repetitions
  
  for nodes to numberOfNodes_max  
    select pattern_train
    plus categories_train
    To FFNet... nodes 0
    ffnet = selected ("FFNet")
    plus pattern_train
    plus categories_train
    Learn... 'number_of_iterations_train' 1e-9 Minimum-squared-error
   
    for j to nspeakers_test
    
      jrow = (i-1)*nspeakers_test + j
      icol = tableoffset + (nodes-1)*3
      
      select ffnet
      
      select speakerindex
      spindex = speaker_test1 - 1 + j
      speakerlabel$ = Get row label... spindex
      from = Get value... spindex 1
      to = Get value... spindex 2
      select table_total
      To Pattern and Categories... 'from' 'to' 0 0
      pattern_test = selected ("Pattern")
      categories_test = selected ("Categories")
    
      call get_fractioncorrect ffnet pattern_test categories_test
      fbefore = fractionCorrect
      
      select result
       
      Set row label (index)... jrow 'speakerlabel$'
      Set value... jrow 1 from
      Set value... jrow 2 to
      Set value... jrow 3 spindex
      Set value... jrow 4 i
      icol += 1
      Set value... jrow icol fbefore
      
      for layer to 2
        select ffnet
        Copy... ffnet_copy
        ffnet_copy = selected ("FFNet")
        Select biases... layer
        if biasreset<> 0
		  Reset... 0.1
		endif
        plus pattern_test
        plus categories_test
        Learn... number_of_iterations_test 1e-7 Minimum-squared-error
        call get_fractioncorrect ffnet_copy pattern_test categories_test
        select result
        icol += 1
        Set value... jrow icol fractionCorrect
        
        select ffnet_copy
        Remove
        call logfile_append rep 'i', nodes 'node', speaker 'j', layer 'layer', fc 'fractionCorrect'
      endfor
      
      select pattern_test
      plus categories_test
      Remove
  
    endfor
  
    select ffnet
    Remove
  endfor
endfor

call logfile_close

select result
Write to short text file... 'savedir$'bias-adaption_'speakerstring$'_'date$'.TableOfReal


